require("defines")

-- Returns the array of distance bonuses based on the given base value.
local function get_distance_bonuses_based_on_value(base_value)
	local bonuses = {}
	bonuses[1] = base_value * 0.5
	bonuses[2] = base_value
	bonuses[3] = base_value * 2
	bonuses[4] = base_value * 4
	return bonuses
end
-- Reach distance bonuses.
local reach_distance_bonuses = get_distance_bonuses_based_on_value(6)

---------------------------------------------------------

-- Sets the reach distance for the given force according to the given long reach research level.
local function set_reach_distance(force, level)
	force.character_reach_distance_bonus = reach_distance_bonuses[level]
	force.character_item_drop_distance_bonus = reach_distance_bonuses[level]
end

-- Sets the build distance for the given force according to the given long build research level.
local function set_build_distance(force, level)
	force.character_build_distance_bonus = reach_distance_bonuses[level]
end

-- Look up table for mapping function for each researched technology name.
local research_finished_functions_lookup =
{
	-- Long reach.
	[long_reach_research.names.technology.long_reach_1] = function(force) set_reach_distance(force, 1) end,
	[long_reach_research.names.technology.long_reach_2] = function(force) set_reach_distance(force, 2) end,
	[long_reach_research.names.technology.long_reach_3] = function(force) set_reach_distance(force, 3) end,
	[long_reach_research.names.technology.long_reach_4] = function(force) set_reach_distance(force, 4) end,
	-- Long build.
	[long_reach_research.names.technology.long_build_1] = function(force) set_build_distance(force, 1) end,
	[long_reach_research.names.technology.long_build_2] = function(force) set_build_distance(force, 2) end,
	[long_reach_research.names.technology.long_build_3] = function(force) set_build_distance(force, 3) end,
	[long_reach_research.names.technology.long_build_4] = function(force) set_build_distance(force, 4) end,
}

-- Sets the team reach distance after a long reach research has completed.
script.on_event(defines.events.on_research_finished, function(event)
	local func = research_finished_functions_lookup[event.research.name]
	if func then
		func(event.research.force)
	end
end)

---------------------------------------------------------

-- Returns the level of long reach research in the given force.
local function get_long_reach_research_level_in_force(force)
	local tech = force.technologies[long_reach_research.names.technology.long_reach_4]
	if tech and tech.researched then
		return 4
	end
	tech = force.technologies[long_reach_research.names.technology.long_reach_3]
	if tech and tech.researched then
		return 3
	end
	tech = force.technologies[long_reach_research.names.technology.long_reach_2]
	if tech and tech.researched then
		return 2
	end
	tech = force.technologies[long_reach_research.names.technology.long_reach_1]
	if tech and tech.researched then
		return 1
	end
	return 0
end

-- Resets build distance, mine distance, item pickup distance and loot pickup distance if this mod is updated to v0.2.0+.
-- Also ensures the reach distance is correct.
script.on_configuration_changed(function(event)
	local change_on_this_mod = event.mod_changes[long_reach_research.mod_id]
	if change_on_this_mod then
		if change_on_this_mod.old_version and change_on_this_mod.old_version < "0.2.0" then
			if change_on_this_mod.new_version and change_on_this_mod.new_version >= "0.2.0" then
				for _, force in pairs(game.forces) do
					force.character_resource_reach_distance_bonus = 0
					force.character_item_pickup_distance_bonus = 0
					force.character_loot_pickup_distance_bonus = 0
					force.character_build_distance_bonus = 0
					
					local level = get_long_reach_research_level_in_force(force)
					if level > 0 then
						set_reach_distance(force, level)
					end
				end
				-- Also free memory. We don't need it anymore.
				global = nil
			end
		end
	end
end)